/* ============================================================
 * File  : mythtubescreen.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-18
 * Description :
 *
 * Generic mythtube interface
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef MYTHTUBESCREEN_CPP
#define MYTHTUBESCREEN_CPP

// C Headers
#include <unistd.h>
#include <math.h>

// QT headers
#include <qapplication.h>
#include <q3network.h>
#include <qdir.h>

//Added by qt3to4:
#include <QUrl>

// MythTV headers
#include <mythtv/libmythui/mythmainwindow.h>
#include <mythtv/util.h>
#include <mythtv/httpcomms.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythdirs.h>

using namespace std;

#include "mythtubescreen.h"

#include <qfile.h>
#include <qstringlist.h>
#include <qdatastream.h>


/**
 * @param  parent
 * @param  name
 */
 MythTubeScreen::MythTubeScreen (MythScreenStack* parent, const char* name )
    : MythScreenType (parent, name)
{
    m_httpGrabber = NULL;
    m_progressPopup = NULL;

    m_cache.size = 4;
    m_cache.index = 0;
}


/**
 */
MythTubeScreen::~MythTubeScreen ( ) {

}


/**
 * @return bool
 */
// bool MythTubeScreen::Create ( ) {
// 
// }


/**
 * @return bool
 * @param  event
 */
bool MythTubeScreen::keyPressEvent (QKeyEvent* event ) {
    if (GetFocusWidget()->keyPressEvent(event))
        return true;

    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Tube", event, actions);

    for (int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];
        handled = true;
        if (action == "ESCAPE")
        {

            if (m_progressPopup)
            {
                m_progressPopup->Close();
                m_progressPopup = NULL;
                m_abortHttp = true;
            }else
                GetScreenStack()->PopScreen();

// esto no esta del todo bien pero a ver si no peta, cuando distribulla las descargas de forma multiple se quitara
//
//             if (m_progressPopup)
//             {
//                 m_progressPopup->Close();
//                 m_progressPopup = NULL;
//             }
// 
// 
//             if (m_httpGrabber)
//                 m_abortHttp = true;
//             else
//                 GetScreenStack()->PopScreen();


        }else{
            handled = false;
        }
    }

    if (!handled && MythScreenType::keyPressEvent(event))
        handled = true;

    return handled;
}


/**
 * @return QString
 * @param  path
 */
QString MythTubeScreen::configDir (QString path) {

    QString fileprefix = GetConfDir();

    QDir dir(fileprefix);
    if (!dir.exists())
        dir.mkdir(fileprefix);
    fileprefix += "/MythTube";
    dir = QDir(fileprefix);
    if (!dir.exists())
        dir.mkdir(fileprefix);

    if(!path.isEmpty()){
        QStringList dir_list = QStringList::split("/" ,path);
        for(int i = 0; i < dir_list.count(); i++){
            fileprefix += "/" + dir_list[i];
            QDir dir_aux(fileprefix);
            if (!dir_aux.exists())
                dir_aux.mkdir(fileprefix);
        }
    }

    return fileprefix;
}


/**
 * @return bool
 * @param  video
 * @param  path
 */
bool MythTubeScreen::downloadVideo (TubeVideo* video, QString path) {

    bool success = false;

    QString cmdURL(video->enclosure());

    QString fileprefix = configDir();

    QString sFilename(fileprefix + "/videotempfile");

    sFilename += QString::number(m_cache.index);

    VERBOSE(VB_GENERAL, QString("downloading on ")+sFilename);

    if (getHttpFile(sFilename, cmdURL))
    {
        success = true;

        m_cache.content.remove(m_cache.content.key(sFilename));
        m_cache.content.insert(video->tubeURL(),sFilename);
        m_cache.index++;
        m_cache.index = m_cache.index % m_cache.size;

        if(!path.isEmpty()){
            QFile file(sFilename);
            file.copy(path + "/" + video->title());
        }

        qApp->unlock();
        playVideo(sFilename);
        qApp->lock();
    }

    return success;
}


/**
 * @return bool
 * @param  sFilename
 * @param  cmdURL
 */
bool MythTubeScreen::getHttpFile (QString sFilename, QString cmdURL ) {

    int redirectCount = 0;
    int timeoutCount = 0;
    QByteArray data(0);
    bool res = false;
//     m_httpGrabber = NULL;
    QString hostname = "";

    VERBOSE(VB_IMPORTANT, cmdURL);

    createProgress(QObject::tr("Downloading media..."));
    while (1)
    {
        QUrl qurl(cmdURL);
        if (hostname == "")
            hostname = qurl.host();  // hold onto original host

        if (!qurl.hasHost())        // can occur on redirects to partial paths
            qurl.setHost(hostname);

        if (m_httpGrabber != NULL){
            m_abortHttp = true;
            m_httpGrabber->stop();
//             delete m_httpGrabber;
        }


        m_httpGrabber = new HttpComms;
        m_abortHttp = false;

        m_httpGrabber->request(qurl, -1, true);

        while ((!m_httpGrabber->isDone()) && (!m_abortHttp))
        {
            int total = m_httpGrabber->getTotal();
            m_progressPopup->SetTotal(total);
            int progress = m_httpGrabber->getProgress();
            m_progressPopup->SetProgress(progress);
            qApp->processEvents();
            usleep(100000);
        }

        if (m_abortHttp)
            break;

        VERBOSE(VB_IMPORTANT, "finish");

        // Check for redirection
        if (!m_httpGrabber->getRedirectedURL().isEmpty())
        {
            if (redirectCount++ < 5)
                cmdURL = m_httpGrabber->getRedirectedURL();

            // Try again
            timeoutCount = 0;
            continue;
        }

        data = m_httpGrabber->getRawData();

        if (data.size() > 0)
        {
            QFile file(sFilename);
            if (file.open( QIODevice::WriteOnly ))
            {
                QDataStream stream(& file);
                stream.writeRawBytes( (const char*) (data), data.size() );
                file.close();
                res = true;
            }
        }
        break;
    }

    if (m_progressPopup)
    {
        m_progressPopup->Close();
        m_progressPopup = NULL;
    }

//     delete m_httpGrabber;
//     m_httpGrabber = NULL;
    return res;
}

/**
 * @param  title
 */
void MythTubeScreen::createProgress (QString title ) {
    if (m_progressPopup)
        return;

    QString message = title;

    MythScreenStack *popupStack = GetMythMainWindow()->GetStack("popup stack");

    m_progressPopup = new MythUIProgressDialog(message, popupStack,
                                               "mythtubeprogressdialog");

    if (m_progressPopup->Create())
        popupStack->AddScreen(m_progressPopup, false);
}


/**
 * @param  filename
 */
void MythTubeScreen::playVideo (QString filename ) {

    QString command_string = gContext->GetSetting("VideoDefaultPlayer");

    gContext->sendPlaybackStart();

    if ((command_string.find("Internal", 0, false) > -1) ||
        (command_string.length() < 1))
    {
        command_string = "Internal";
        gContext->GetMainWindow()->HandleMedia(command_string, filename);
    }
    else
    {
        if (command_string.contains("%s"))
            command_string = command_string.replace(QRegExp("%s"), filename);

        myth_system(command_string);
    }

    gContext->sendPlaybackEnd();

}


/**
 * @param  message
 */
void MythTubeScreen::showInfo (QString message ) {
    QString label = tr(message);

    MythScreenStack *popupStack =
                            GetMythMainWindow()->GetStack("popup stack");

    m_infoPopup = new MythDialogBox(label, popupStack, "mythtubeinfopopup");

    if (m_infoPopup->Create())
        popupStack->AddScreen(m_infoPopup);
}


void MythTubeScreen::clearInfo (){
    if (m_infoPopup){
        m_infoPopup->Close();
        m_infoPopup = NULL;
    }
}


bool MythTubeScreen::onCache(TubeVideo *video){
    return m_cache.content.contains(video->tubeURL());
}

QString MythTubeScreen::getCache(TubeVideo *video){

    QString file;

    if(m_cache.content.contains(video->tubeURL()))
        file = m_cache.content.value(video->tubeURL());

    return file;

}

void MythTubeScreen::downloadVideo2 (TubeVideo* video) {

    while(1){
        VERBOSE(VB_IMPORTANT, video->title());
        qApp->processEvents();
        usleep(1000000);
    }
}

#endif
