#!/usr/bin/perl -w

#
# This script search and get video data from youtube
# and return a xml archive
#
# This is a test script ident to youtube.pl but order video list
# by published date
#
# Author: Alvaro Gonzalez Sola (cojonuo@gmail.com)
# v0.8
#
# changes:
#
#   v0.1 (08/08/2008) (Alvaro Gonzalez Sola):
#         - initial version
#
#   v0.8 (10/08/2008) (Alvaro Gonzalez Sola):
#         - add new function getMediaURL (with -M option), get the real url of media to get it
#           this function is separated to search beacuse usually take more time and do it for all
#           video list is too slow. This option need video link and return this video link and 
#           media url.
#         - change output format: now use <entry> to limit video items.
#         - add new info: now script get more info for video, exactly: published date, duration
#           (seconds) and view count.
#
#   v0.9 (21/08/2008)
#         - clean some comment line
#         - change es.youtube.com to www.youtube.com
#

use LWP::UserAgent;
use URI::Escape;
use XML::Parser;
use XML::DOM;

use Encode;

eval "use DateTime::Format::Strptime"; my $has_date_format = $@ ? 0 : 1;

use vars qw($opt_h $opt_r $opt_d $opt_i $opt_v $opt_S $opt_M);
use Getopt::Std; 

$title = "Youtube search"; 
$version = "v0.8";
$author = "Alvaro Gonzalez Sola (cojonuo\@gmail.com)";

binmode(STDOUT, ":utf8");

# display usage
sub usage {
   print "usage: $0 -hdrviMPD [parameters]\n";
   print "       -h           help\n";
   print "       -d           debug\n";
   print "       -r           dump raw query result data only\n";
   print "       -v           display version\n";
   print "       -i           display info\n";
   print "\n";
   print "       -M <query>    search videos\n";
   exit(-1);
}

# display 1-line of info that describes the version of the program
sub version {
   print "$title ($version) by $author\n"
} 

# display 1-line of info that can describe the type of query used
sub info {
   print "Search videos on youtube.\n";
}

# display detailed help 
sub help {
   version();
   info();
   usage();
}

# search and dump result on xml format
sub search {
   my ($filename)=@_; # grab parameters # adquiere los parametros

   # If we wanted to inspect the file for any reason we can do that now

   #
   # Convert filename into a query string 
   # (use same rules that Metadata::guesTitle does)

   my $query = $filename;
   $query = uri_unescape($query);  # in case it was escaped

   # Strip off the file extension
   if (rindex($query, '.') != -1) {
      $query = substr($query, 0, rindex($query, '.'));
   }

   # Strip off anything following '(' - people use this for general comments
   if (rindex($query, '(') != -1) {
      $query = substr($query, 0, rindex($query, '(')); 
   }

   # Strip off anything following '[' - people use this for general comments
   if (rindex($query, '[') != -1) {
      $query = substr($query, 0, rindex($query, '[')); 
   }

   # prepare the url
   $query = uri_escape($query);
   if (defined $opt_d) { 
      printf("# query: '%s'\n", $query);
   }

   # get the search results  page
   my $request="http://gdata.youtube.com/feeds/api/videos?vq=" . $query ."&orderby=published";
   if (defined $opt_d) { printf("# request: '%s'\n", $request); }

   my $parser = new XML::DOM::Parser;
   my $doc = $parser->parsefile ($request);
   if (defined $opt_r) { printf("# XML Document: '%s'\n", $doc->toString); }

#    my $nodes = $doc->getElementsByTagName ("media:group");
   my $nodes = $doc->getElementsByTagName ("entry");
   my $n = $nodes->getLength;

   print "<?xml version='1.0' encoding='UTF-8'?><feed>";

   for (my $i = 0; $i < $n; $i++)
   {
      my $node = $nodes->item ($i);

      my $title = $node->getElementsByTagName ("media:title");
      $title = $title->item(0);
      my $title_cont = "";
      if ($title) { $title_cont = ($title)->getFirstChild; }

      my $date = $node->getElementsByTagName ("published");
      $date = $date->item(0);
      my $date_cont = "";
      if ($date) { $date_cont = ($date)->getFirstChild; }
      $date_cont->toString =~ m/(.*)-(.*)-(.*)T.*/;
      $date_cont = $3 . "-" . $2 . "-" . $1;

      my $description = $node->getElementsByTagName ("media:description");
      $description = $description->item(0);
      my $description_cont = "";
      if ($description) { $description_cont = ($description)->getFirstChild; }

      my $url = $node->getElementsByTagName ("media:player");
      $url = $url->item(0);
      my $url_cont = "";
      if ($url) { $url_cont = $url->getAttribute("url"); }

      my $thumbnail = $node->getElementsByTagName ("media:thumbnail");
      $thumbnail = $thumbnail->item($thumbnail->getLength -1);
      my $thumbnail_cont = "";
      if ($thumbnail) { $thumbnail_cont = $thumbnail->getAttribute("url"); }

      my $time = $node->getElementsByTagName ("yt:duration");
      $time = $time->item(0);
      my $time_cont = "";
      if ($time) { $time_cont = $time->getAttribute("seconds"); }

      my $viewcount = $node->getElementsByTagName ("yt:statistics");
      $viewcount = $viewcount->item(0);
      my $viewcount_cont = "";
      if ($viewcount) { $viewcount_cont = $viewcount->getAttribute("viewCount"); }



      my $mediaURL = $url;
      my $mediaURL_cont = $url_cont;

#       my $enclosureURL = $node->getElementsByTagName ("media:content");
#       $enclosureURL = $enclosureURL->item(0);
#       my $enclosureURL_cont = "";
#       if ($enclosureURL) { $enclosureURL_cont = $enclosureURL->getAttribute("url"); }
# 
#       $mediaURL_cont =~ m/http:\/\/www.youtube.com\/watch\?v=(.*)/;
#       my $videoID = $1;
# 
#       if($enclosureURL_cont =~ m/rtsp.*/){
#          $enclosureURL_cont = "http://www.youtube.com/v/".$videoID;
#       }
# 
#       # get the search results  page
#       my $ua = LWP::UserAgent->new;
#       $ua->timeout(10);
#       $ua->env_proxy;
# 
#       my $response = $ua->post($enclosureURL_cont);
# 
#       my $location = $response->as_string."\n";
#       $location =~ m/Location:(.*)/g;
#       $location = $1;
#       $location =~ m/&t=(.*)&/g;
#       my $sesionID = $1;
# 
#       $enclosureURL_cont = "http://es.youtube.com/get_video.php?video_id=".$videoID."&amp;t=".$sesionID."&amp;fmt=18";

      print "<entry>\n";
      print "  <title>" . $title_cont->toString . "</title>\n";
      print "  <date>" . $date_cont . "</date>\n";
      print "  <content>" . $description_cont->toString . "</content>\n";
      print "  <link href='" . $url_cont . "'/>\n";
      print "  <media:group>\n";
      print "     <media:thumbnail url='" . $thumbnail_cont . "'/>\n";
      print "     <media:player url='" . $mediaURL_cont . "'/>\n";
      print "     <media:time sec='" . $time_cont . "'/>\n";
#       print "     <media:content url='" . $enclosureURL_cont . "'/>\n";
      print "  </media:group>\n";
      print "  <rating viewCount='" . $viewcount_cont . "'/>\n";
      print "</entry>\n";

   }
   print "</feed>";
}

# get video page link and return this.
sub getMediaURL {
   my ($filename)=@_; # grab parameters
   my $page = $filename;

   # get the search results  page
   my $ua = LWP::UserAgent->new;
   $ua->timeout(10);
   $ua->env_proxy;

   $page =~ m/.*\/watch\?v=(.*)/;
   my $videoID = $1;
   my $url = "http://www.youtube.com/v/".$videoID;

   my $response = $ua->post($url);

   my $location = $response->as_string."\n";
   $location =~ m/Location:(.*)/g;
   $location = $1;
   $location =~ m/&t=(.*)&/g;
   my $sesionID = $1;

   my $enclosureURL_cont = "http://es.youtube.com/get_video.php?video_id=".$videoID."&t=".$sesionID."&fmt=18";
   print $page . " " . $enclosureURL_cont;
}


#
# Main Program
#

# parse command line arguments 
getopts('hrdviSM');

# print out info 
if (defined $opt_v) { version(); exit 1; }
if (defined $opt_i) { info(); exit 1; }

# print out usage if needed
if (defined $opt_h || $#ARGV<0) { help(); }

# search
elsif (defined $opt_S) {
   # take query from cmdline arg
   $options = shift || die "Usage : $0 -S <query>\n";
   $query = shift;
   if (!$query) {
      $query = $options;
      $options = "";
   }
   search($query);
}
# get media url
elsif (defined $opt_M) {
   # take video page link
   $options = shift || die "Usage : $0 -M <video_page>\n";
   $page = shift;
   if (!$page) {
      $page = $options;
      $options = "";
   }
   getMediaURL($page);
}
