/* ============================================================
 * File  : tubedbutil.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-18
 * Description :
 *
 * Utils to DB
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef TUBEDBUTIL_CPP
#define TUBEDBUTIL_CPP

#include <mythtv/mythdbcon.h>
#include <mythtv/mythcontext.h>

#include "tubedbutil.h"
#include "feedchannel.h"
#include "tubevideo.h"

ChannelBrowser::ChannelList loadChannels(){

    ChannelBrowser::ChannelList channels;

    MSqlQuery query(MSqlQuery::InitCon());
    query.exec("SELECT name, url, type FROM tubechannel ORDER BY name");

    if (!query.isActive()) {
        VERBOSE(VB_IMPORTANT, "MythTube: Error in loading Channels from DB");
    }
    else {
        QString name;
        QString url;
        QString type;
        while ( query.next() ) {
            name = query.value(0).toString();
            url  = query.value(1).toString();
            type = query.value(2).toString();

            if(type == "Feed"){
                channels.append(new FeedChannel(name,url));
            }
            else{
                channels.append(new Channel(name,url,Channel::Search));
            }
        }
    }

    return channels;
}

bool findInDB(const QString& name)
{
    MSqlQuery query(MSqlQuery::InitCon());
    query.prepare("SELECT name FROM tubechannel WHERE name = :NAME ;");
    query.bindValue(":NAME", name);
    if (!query.exec() || !query.isActive()) {
        MythContext::DBError("MythTube find in db", query);
        return false;
    }

    return query.size() > 0;
}

bool insertInDB(QString name, QString url, Channel::Type type){
    if (findInDB(name))
        return false;

    QString type_text;
    if(type == Channel::Search)
        type_text = "Search";
    else
        type_text = "Feed";

    MSqlQuery query(MSqlQuery::InitCon());
    query.prepare("INSERT INTO tubechannel (name,url,type) "
            " VALUES( :NAME, :URL, :TYPE );");
    query.bindValue(":NAME", name);
    query.bindValue(":URL", url);
    query.bindValue(":TYPE", type_text);
    if (!query.exec() || !query.isActive()) {
        MythContext::DBError("MythTube: inserting in DB", query);
        return false;
    }

    return (query.numRowsAffected() > 0);

}

bool insertInDB(Channel * channel){
    if(! channel)
        return false;

    return insertInDB(channel->name(),channel->url(),channel->type());
}

bool removeFromDB(Channel * channel){
    if (!channel) return false;

    return removeFromDB(channel->name());
}

bool removeFromDB(const QString &name)
{
    MSqlQuery query(MSqlQuery::InitCon());
    query.prepare("DELETE FROM tubechannel WHERE name = :NAME ;");
    query.bindValue(":NAME", name);
    if (!query.exec() || !query.isActive()) {
        MythContext::DBError("MythTube: delete from db", query);
        return false;
    }

    return (query.numRowsAffected() > 0);
}

#endif