/* ============================================================
 * File  : tubevideo.h
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-07-30
 * Description :
 *
 * Search engine to video search sites using external script.
 * And TubeVideo content class for videos
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef TUBEVIDEO_CPP
#define TUBEVIDEO_CPP

// #include <iostream>

#include <qfile.h>
#include <qdatastream.h>
#include <qdom.h>
#include <q3urloperator.h>
#include <q3network.h>
#include <q3process.h>

#include "tubevideo.h"

#include <mythtv/mythcontext.h>
#include <mythtv/mythdirs.h>

using namespace std;


//========================================================================================
//          TubeVideo Construction, Destruction
//========================================================================================

TubeVideo::TubeVideo(const QString& title, const QString& desc,
              const QString& tubeURL, const QString& thumbnail,
              const QString& mediaURL, const QString& enclosure,
              const QString& date, const QString& time, 
              const QString& rating){

    m_title = title;
    m_desc = desc;
    m_tubeURL = tubeURL;
    m_thumbnail = thumbnail;
    m_mediaURL = mediaURL;
    m_enclosure = enclosure;
    m_date = date;
    m_time = time;
    m_rating = rating;
}

TubeVideo::~TubeVideo(){

}



//========================================================================================
//          Search Construction, Destruction
//========================================================================================

Search::Search(const QString& query, const QString& script){

    m_destDir  = GetConfDir();
    m_destDir += "/MythTube";
    m_name    = "MythTube_temp";

    m_query = query;
    m_script = script;
    m_searchProcess = new Q3Process();
    m_moreInfoProcess = new Q3Process();

    m_videoList.setAutoDelete(true);
    m_videoList.clear();

    connect(m_searchProcess, SIGNAL(processExited()),
                  SLOT(slotProcessSearchExit()));
    connect(m_moreInfoProcess, SIGNAL(processExited()),
                  SLOT(slotProcessMoreInfoExit()));
}

Search::~Search(){
    VERBOSE(VB_IMPORTANT, "matando script");
    m_videoList.clear();
}


//========================================================================================
//          Public Function
//========================================================================================

void Search::executeSearch(){

   QString query = "\""+m_query+"\"";

   QString cmd = QDir::cleanDirPath(QString("%1/%2")
                    .arg(GetShareDir())
                    .arg("mythtube/scripts/"+m_script+" -S"));

   

   QStringList args;
   args += query;

   QStringList split_args = QStringList::split(' ', cmd);
   split_args += args;

   m_searchProcess->clearArguments();
   m_searchProcess->setArguments(split_args);

   VERBOSE(VB_IMPORTANT, "Launch: "+cmd);

   if(!m_searchProcess->start()){
      VERBOSE(VB_IMPORTANT, "ERROR: Script");
   }

}

void Search::getMoreInfo(TubeVideo* item){

   VERBOSE(VB_IMPORTANT, "MythTube: TubeEngine: Obteniendo mas info");

   QString url = item->tubeURL();
   QString cmd = QDir::cleanDirPath(QString("%1/%2")
                    .arg(GetShareDir())
                    .arg("mythtube/scripts/"+m_script+" -M "));

    VERBOSE(VB_IMPORTANT, "MythTube: TubeEngine: " + cmd);

   QStringList args;
   args += url;

   QStringList split_args = QStringList::split(' ', cmd);
   split_args += args;

   m_moreInfoProcess->clearArguments();
   m_moreInfoProcess->setArguments(split_args);

   if(!m_moreInfoProcess->start()){
      VERBOSE(VB_IMPORTANT, "ERROR: Script");
   }

}

void Search::process(){
    QDomDocument domDoc;

    QFile xmlFile(m_destDir+QString("/")+m_name);
    if (!xmlFile.exists()) {
        return;
    }

    if (!xmlFile.open(QIODevice::ReadOnly)) {
        VERBOSE(VB_IMPORTANT, "MythTube: TubeEngine: failed to open xmlfile");
        return;
    }

    if (!domDoc.setContent(&xmlFile)) {
        VERBOSE(VB_IMPORTANT, "MythTube: TubeEngine: failed to set content from xmlfile");
        return;
    }

    m_videoList = parseAtom(domDoc);

    xmlFile.close();
}

TubeVideo::List Search::parseAtom(QDomDocument domDoc)
{
    TubeVideo::List vList;

    QDomNodeList entries = domDoc.elementsByTagName(QString::fromLatin1("entry"));

    QDomNode itemNode;
    QString title, description, url, thumbnail, mediaurl, enclosure, imageURL, date, time, rating;

    for (int i = 0; i < entries.count(); i++) {
        itemNode = entries.item(i);
        title    = itemNode.namedItem(QString::fromLatin1("title")).toElement().text().simplifyWhiteSpace();
        if (!title.isNull())
            ReplaceHtmlChar(title);

        QDomNode summNode = itemNode.namedItem(QString::fromLatin1("content"));
        if (!summNode.isNull())
        {
            description = summNode.toElement().text().simplifyWhiteSpace();
            ReplaceHtmlChar(description);
        }
        else
            description = QString::null;

        QDomNode linkNode = itemNode.namedItem(QString::fromLatin1("link"));
        if (!linkNode.isNull()){
            QDomAttr linkHref = linkNode.toElement().attributeNode("href");
            if(!linkHref.isNull())
                url = linkHref.value();
        }
        else
            url = QString::null;

        QDomNode dateNode = itemNode.namedItem(QString::fromLatin1("date"));
        if (!dateNode.isNull())
        {
            date = dateNode.toElement().text().simplifyWhiteSpace();
        }
        else
            date = QString::null;

       QDomNode ratingNode = itemNode.namedItem(QString::fromLatin1("rating"));
        if (!ratingNode.isNull()){
            QDomAttr viewCount = ratingNode.toElement().attributeNode("viewCount");
            if(!viewCount.isNull())
                rating = viewCount.value();
        }
        else
            rating = QString::null;

        // From this point forward, we process RSS 2.0 media tags.  Please put all
        // other tag processing before this
        // Some media: tags can be enclosed in a media:group item.  If this item is
        // present, use it to find the media tags, otherwise, proceed.
        QDomNode mediaGroup = itemNode.namedItem(QString::fromLatin1("media:group"));
            if (!mediaGroup.isNull())
                itemNode = mediaGroup;

        QDomNode thumbNode = itemNode.lastChildElement(QString::fromLatin1("media:thumbnail"));
        if (!thumbNode.isNull())
        {
            QDomAttr thumburl = thumbNode.toElement().attributeNode("url");
            if (!thumburl.isNull())
                thumbnail = thumburl.value();
        } else
            thumbnail = QString::null;

        QDomNode playerNode = itemNode.namedItem(QString::fromLatin1("media:player"));
        if (!playerNode.isNull())
        {
            QDomAttr mediaURL = playerNode.toElement().attributeNode("url");
            if (!mediaURL.isNull())
               mediaurl  = mediaURL.value();
        } else
            mediaurl = QString::null;

        QDomNode contentNode = itemNode.namedItem(QString::fromLatin1("media:content"));
        if (!contentNode.isNull())
        {
            QDomAttr enclosureURL = contentNode.toElement().attributeNode("url");
            if (!enclosureURL.isNull())
                enclosure  = enclosureURL.value();
            else
                enclosure = QString::null;
        }

        QDomNode timeNode = itemNode.namedItem(QString::fromLatin1("media:time"));
        if (!timeNode.isNull())
        {
            QDomAttr sec = timeNode.toElement().attributeNode("sec");
            if (!sec.isNull())
                time  = sec.value();
            else
                time = QString::null;
        }

        vList.append(new TubeVideo(title, description, url, thumbnail, mediaurl, enclosure, date, time, rating));

    }

    return vList;
}


//========================================================================================
//          Manage video list
//========================================================================================


TubeVideo::List& Search::videoList()
{
    return m_videoList;
}




//========================================================================================
//          Slots
//========================================================================================


void Search::slotProcessSearchExit(){

    VERBOSE(VB_IMPORTANT, "MythTube: TubeVideo: Finish Search script execution");

    m_data = m_searchProcess->readStdout();

    QFile xmlFile(m_destDir+QString("/")+m_name);
    if (xmlFile.open( QIODevice::WriteOnly )) {
        QDataStream stream( &xmlFile );
        stream.writeRawBytes(m_data.data(), m_data.size());
        xmlFile.close();
        m_state = Search::Success;
    }
    else 
    {
        m_state = Search::WriteFailed;
        VERBOSE(VB_IMPORTANT, "MythTube: TubeVideo: Write failed");
    }

    emit finishedSearch(this);
}

void Search::slotProcessMoreInfoExit(){

    VERBOSE(VB_IMPORTANT, "MythTube: TubeVideo: Finish MoreInfo script execution");

    QString out = m_moreInfoProcess->readStdout();

    QStringList list_out = out.split(" ");

    QString link = list_out[0];
    QString url = list_out[1];

    TubeVideo * video_aux;

    for (TubeVideo* video = m_videoList.first(); video; video = m_videoList.next()) {
        if(video->tubeURL()==link){

            video->setEnclosure(url);

            video_aux = video;
        }
    }


    VERBOSE(VB_IMPORTANT, "MythTube: TubeVideo: URL "+video_aux->tubeURL());

    emit finishedMoreInfo(video_aux);
}

//========================================================================================
//          Util
//========================================================================================

void Search::ReplaceHtmlChar(QString &s)
{
    s.replace("&amp;", "&");
    s.replace("&lt;", "<");
    s.replace("&gt;", ">");
    s.replace("&quot;", "\"");
    s.replace("&apos;", "\'");
    s.replace("&#8230;",QChar(8230));
    s.replace("&#233;",QChar(233));
    s.replace("&mdash;", QChar(8212));
    s.replace("&nbsp;", " ");
    s.replace("&#160;", QChar(160));
    s.replace("&#225;", QChar(225));
    s.replace("&#8216;", QChar(8216));
    s.replace("&#8217;", QChar(8217));
    s.replace("&#039;", "\'");
    s.replace("&ndash;", QChar(8211));
    // german umlauts
    s.replace("&auml;", QChar(0x00e4));
    s.replace("&ouml;", QChar(0x00f6));
    s.replace("&uuml;", QChar(0x00fc));
    s.replace("&Auml;", QChar(0x00c4));
    s.replace("&Ouml;", QChar(0x00d6));
    s.replace("&Uuml;", QChar(0x00dc));
    s.replace("&szlig;", QChar(0x00df));
}

#endif
