/* ============================================================
 * File  : tubevideo.h
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-07-30
 * Description :
 *
 * Search engine to video search sites using external script.
 * And TubeVideo content class for videos
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef TUBEVIDEO_H
#define TUBEVIDEO_H

#include <qstring.h>
#include <q3ptrlist.h>
#include <qobject.h>
#include <q3cstring.h>
#include <q3process.h>

class Search;
class QDomDocument;

class TubeVideo
{

public:

    typedef Q3PtrList<TubeVideo> List;

    TubeVideo(const QString& title, const QString& desc,
              const QString& tubeURL, const QString& thumbnail,
              const QString& mediaURL, const QString& enclosure,
              const QString& date, const QString& time, 
              const QString& rating);
    ~TubeVideo();

    const QString& title() const { return m_title; }
    const QString& description() const { return m_desc; }
    const QString& tubeURL() const { return m_tubeURL; }
    const QString& thumbnail() const { return m_thumbnail; }
    const QString& mediaURL() const { return m_mediaURL; }
    const QString& enclosure() const { return m_enclosure; }
    const QString& date() const { return m_date; }
    const QString& time() const { return m_time; }
    const QString& rating() const { return m_rating; }

    void setEnclosure(QString enclosure) { m_enclosure = enclosure; }

private:

    QString   m_title;
    QString   m_desc;
    QString   m_tubeURL;
    QString   m_thumbnail;
    QString   m_mediaURL;
    QString   m_enclosure;
    QString   m_date;
    QString   m_time;
    QString   m_rating;

};


class Search : public QObject
{

    Q_OBJECT

public:

    enum State {
        Retrieving = 0,
        RetrieveFailed,
        WriteFailed,
        Success
    };


    Search(const QString& query,const QString& script);
    ~Search();

    void executeSearch();
    void getMoreInfo(TubeVideo* item);
    void process();
    TubeVideo::List parseAtom(QDomDocument domDoc);

    TubeVideo::List& videoList();

private:

    Q3Process* m_searchProcess;
    Q3Process* m_moreInfoProcess;

    QString        m_query;
    QString        m_script;
    QString        m_name;
    QString        m_destDir;

    State          m_state;
    QByteArray     m_data;
    TubeVideo::List m_videoList;
    void ReplaceHtmlChar(QString &s);
signals:

    void finishedSearch(Search* item);
    void finishedMoreInfo(TubeVideo* video);

private slots:

    void slotProcessSearchExit();
    void slotProcessMoreInfoExit();
};


#endif
